/*  $Id: datafile_inputprovider.h,v 1.14 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_datafile_inputprovider
#define header_datafile_inputprovider

#if _MSC_VER > 1000
#pragma once
#endif

#include <map>
#include <utility>
#include <zlib.h>

#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/inputsource_provider.h"

#include <stack>

class CL_Zipped_Position
{
public:
	gzFile gzfile;
	int datafile_pos;
	int seek_pos;
};

class CL_InputSourceProvider_Datafile;

class CL_InputSource_Datafile : public CL_InputSource
{
protected:
	std::string filename;
	CL_InputSourceProvider_Datafile *provider;

	int objsize;
	gzFile gzfile;
	int datafile_handle;
	int index_open;
	char compressed;
	int index_pos;

	std::stack<CL_Zipped_Position> index_stack;
	
	int seek_pos;

public:
	CL_InputSource_Datafile(const std::string &filename, CL_InputSourceProvider_Datafile *provider);
	CL_InputSource_Datafile(const CL_InputSource_Datafile *source);
	virtual ~CL_InputSource_Datafile();

	//: read larger amounts of data (no endian and 64 bit conversion):
	//return: num bytes actually read
	virtual int read(void *data, int size);

	virtual void open();
	virtual void close();

	//: Make a copy of the current InputSource, standing at the same position.
	virtual CL_InputSource *clone() const;

	//: Returns current position in input source
	virtual int tell() const;
	virtual void seek(int pos, SeekEnum seek_type);

	//: Returns the size of the input source
	virtual int size() const;

	virtual void push_position();
	virtual void pop_position();
};

class CL_InputSourceProvider_Datafile : public CL_InputSourceProvider
{
public:
	CL_InputSourceProvider_Datafile(const std::string &datafile);
	CL_InputSourceProvider_Datafile(CL_InputSourceProvider_Datafile *datafile);
	virtual ~CL_InputSourceProvider_Datafile();

	virtual CL_InputSource *open_source(const std::string &filename);
	virtual CL_InputSourceProvider *clone();
	virtual std::string get_pathname(const std::string &filename);
	virtual CL_InputSourceProvider *create_relative(const std::string &path);

	int get_handle() const { return datafile_handle; }
	bool lookup_resource(const std::string &resource_id, int &pos, int &size) const
	{
		return resource_cache->lookup(resource_id, pos, size);
	}

protected:
	class IndexLocationCache
	{
	public:
		IndexLocationCache() : refcount(1) { ; }

		void addref() { refcount++; }
		void release()
		{
			refcount--;
			if (!refcount) delete this;
		}

		bool lookup(const std::string &resource_id, int &data_pos, int &data_size) const
		{
			std::map<std::string, std::pair<int, int> >::const_iterator it = cache.find(resource_id);
			if (it == cache.end())
				return false;

			data_pos = (*it).second.first;
			data_size = (*it).second.second;

			return true;
		}

		void insert(const std::string &resource_id, int data_pos, int data_size)
		{
			cache.insert(
				std::make_pair<std::string const, std::pair<int, int> >(
				resource_id,
				std::make_pair<int,int>(data_pos, data_size)));
		}

	private:
		int refcount;
		std::map<std::string, std::pair<int, int> > cache;
	};
	IndexLocationCache *resource_cache;

	std::string datafile;
	int datafile_handle;
	void open();
	void load_resource_ids();
};

#endif
