/*  $Id: directory_scanner_generic.h,v 1.13 2004/02/01 02:06:29 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_directory_scanner_generic
#define header_directory_scanner_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include <string>

class CL_DirectoryScanner_Generic
{
//! Construction:
public:
	CL_DirectoryScanner_Generic() { return; }

	virtual ~CL_DirectoryScanner_Generic() { return; }
	
//! Attributes:
public:
	virtual bool scan (const std::string& pathname) =0;

	virtual bool scan (const std::string& pathname, const std::string& pattern) =0;

	//: Returns the path of the directory being scanned.
	virtual std::string get_directory_path() = 0;

	//: Returns the name of the current found file.
	virtual std::string get_name() = 0;

	//: Returns the size of the current found file.
	virtual int get_size() = 0;

	//: Returns the name of the current found file, including the directory path.
	virtual std::string get_pathname() = 0;
	
	//: Returns true if filename is a directory.
	virtual bool is_directory() = 0;

	//: Returns true if filename is hidden.
	virtual bool is_hidden() = 0;

	//: Returns true if file is readable by current user.
	virtual bool is_readable() = 0;

	//: Returns true if file is writable by current user.
	virtual bool is_writable() = 0;

	// todo: add other attributes of a file.

//! Operations:
public:
	//: Find next file in directory scan. Returns false if no more files was found.
	virtual bool next() = 0;

//! Implementation:
private:
};

#endif
