/*  $Id: inputsource.cpp,v 1.6 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/cl_endian.h"

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource operations:

void CL_InputSource::set_system_mode()
{
	little_endian_mode = true; // fix me: should use whatever endianess the system uses.
}

void CL_InputSource::set_big_endian_mode()
{
	little_endian_mode = false;
}

void CL_InputSource::set_little_endian_mode()
{
	little_endian_mode = true;
}

cl_int64 CL_InputSource::read_int64()
{
	cl_int64 answer;
	if (read(&answer, sizeof(cl_int64)) != sizeof(cl_int64)) throw CL_Error("CL_InputSource_Datafile::read_int64() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_uint64 CL_InputSource::read_uint64()
{
	cl_uint64 answer;
	if (read(&answer, sizeof(cl_uint64)) != sizeof(cl_uint64)) throw CL_Error("CL_InputSource_Datafile::read_uint64() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_int32 CL_InputSource::read_int32()
{
	cl_int32 answer;
	if (read(&answer, sizeof(cl_int32)) != sizeof(cl_int32)) throw CL_Error("CL_InputSource_Datafile::read_int32() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_uint32 CL_InputSource::read_uint32()
{
	cl_uint32 answer;
	if (read(&answer, sizeof(cl_uint32)) != sizeof(cl_uint32)) throw CL_Error("CL_InputSource_Datafile::read_uint32() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_int16 CL_InputSource::read_int16()
{
	cl_int16 answer;
	if (read(&answer, sizeof(cl_int16)) != sizeof(cl_int16)) throw CL_Error("CL_InputSource_Datafile::read_int16() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_uint16 CL_InputSource::read_uint16()
{
	cl_uint16 answer;
	if (read(&answer, sizeof(cl_uint16)) != sizeof(cl_uint16)) throw CL_Error("CL_InputSource_Datafile::read_uint16() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_int8 CL_InputSource::read_int8()
{
	cl_int8 answer;
	if (read(&answer, sizeof(cl_int8)) != sizeof(cl_int8)) throw CL_Error("CL_InputSource_Datafile::read_int8() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

cl_uint8 CL_InputSource::read_uint8()
{
	cl_uint8 answer;
	if (read(&answer, sizeof(cl_uint8)) != sizeof(cl_uint8)) throw CL_Error("CL_InputSource_Datafile::read_uint8() failed");
	SWAP_IF_BIG(answer);
	return answer;
}

short CL_InputSource::read_short16()
{
	short answer;
	if (read(&answer, sizeof(short)) != sizeof(short)) throw CL_Error("CL_InputSource_Datafile::read_short16() failed");

	SWAP_IF_BIG(answer);
	// todo: do endianess converting here, if needed.
	return answer;
}

unsigned short CL_InputSource::read_ushort16()
{
	unsigned short answer;
	if (read(&answer, sizeof(unsigned short)) != sizeof(unsigned short)) throw CL_Error("CL_InputSource_Datafile::read_ushort16() failed");

	SWAP_IF_BIG(answer);
	// todo: do endianess converting here, if needed.
	return answer;
}

char CL_InputSource::read_char8()
{
	char answer;
	if (read(&answer, sizeof(char)) != sizeof(char)) throw CL_Error("CL_InputSource_Datafile::read_char8() failed");
	// todo: do endianess converting here, if needed.
	return answer;
}

unsigned char CL_InputSource::read_uchar8()
{
	unsigned char answer;
	if (read(&answer, sizeof(unsigned char)) != sizeof(unsigned char)) throw CL_Error("CL_InputSource_Datafile::read_uchar8() failed");
	// todo: do endianess converting here, if needed.
	return answer;
}

float CL_InputSource::read_float32()
{
	float answer;
	if (read(&answer, sizeof(float)) != sizeof(float)) throw CL_Error("CL_InputSource_Datafile::read_float32() failed");


	SWAP_IF_BIG(answer);
	
	return answer;
}

bool CL_InputSource::read_bool8()
{
	bool answer;
	if (read(&answer, sizeof(bool)) != sizeof(bool)) throw CL_Error("CL_InputSource_Datafile::read_bool8() failed");
	// todo: do endianess converting here, if needed.
	return answer;
}

std::string CL_InputSource::read_string()
{
	int size = read_int32();

	char *str = new char[size];
	try
	{
		read(str, size);
		
		std::string ret(str, size);
		delete[] str;

		return ret;
	}
	catch (...)
	{
		delete[] str;
		throw;
	}
}
