/*  $Id: inputsource_memory_generic.h,v 1.10 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_inputsource_memory_generic
#define header_inputsource_memory_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/inputsource_provider.h"

class CL_OutputSource_MemoryGeneric;

class CL_InputSource_MemoryGeneric : public CL_InputSource
//: Interface to read data from a memory_generic source.
//- <p>CL_InputSource_MemoryGeneric is used to read data from a memory_generic source.</p>
//also: CL_InputSourceProvider - Interface to open input sources with.
{
public:
	CL_InputSource_MemoryGeneric(void *data, int size, bool delete_data = false);

	//: MemoryGeneric copy constructor.
	//- <p>Makes a seperate copy of the memory in MG.</p>
	//also: MemoryGeneric::clone.
	//param MG: Pointer to the MemoryGeneric object from which to copy.
	CL_InputSource_MemoryGeneric(const CL_InputSource_MemoryGeneric *MG);

	virtual ~CL_InputSource_MemoryGeneric();

	//: Reads larger amounts of data (no endian and 64 bit conversion).
	//param data: Points to an array where the read data is stored.
	//param size: Number of bytes to read.
	//return: Num bytes actually read.
	virtual int read(void *data, int size);
	
	//: Opens the input source. By default, it is open.
	virtual void open();
	
	//: Closes the input source.
	virtual void close();

	//: Make a copy of the current InputSource, standing at the same position.
	//return: The clone of the input source.
	virtual CL_InputSource *clone() const;

	//: Returns current position in input source.
	//return: Current position in input source.
	virtual int tell() const;

	//: Seeks to the specified position in the input source.
	//param pos: Position relative to 'seek_type'.
	//param seek_type: Defines what the 'pos' is relative to. Can be either seek_set, seek_cur og seek_end.
	virtual void seek(int pos, SeekEnum seek_type);

	//: Returns the size of the input source
	//return: Size of the input source.
	virtual int size() const;

	//: Pushes the current input source position. The position can be restored again with pop_position.
	virtual void push_position();
	
	//: Pops a previous pushed input source position (returns to the position).
	virtual void pop_position();

	//: Purges the input buffer of data without deleting the buffer
	virtual void purge();

private:
	unsigned char *data;
	unsigned int pos, length;
	bool delete_data;
};

class CL_InputSourceProvider_Memory : public CL_InputSourceProvider
{
public:
	CL_InputSourceProvider_Memory(unsigned char *data, unsigned int size, bool delete_data);
	CL_InputSourceProvider_Memory(CL_InputSource_MemoryGeneric *MG);
	CL_InputSourceProvider_Memory(CL_OutputSource_MemoryGeneric *MG);

	virtual CL_InputSource *open_source(const std::string &handle);
	virtual CL_InputSourceProvider *clone();

private:
	unsigned char *data;
	unsigned int size;
	bool delete_data;
};

#endif

