/*  $Id: inputsource_zipped.cpp,v 1.4 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include "API/Core/IOData/inputsource_zipped.h"

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Zipped_Generic

class CL_InputSource_Zipped_Generic
{
public:
	CL_InputSource *input;
	bool delete_input;
};

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Zipped Construction:

CL_InputSource_Zipped::CL_InputSource_Zipped(
	CL_InputSource *input,
	bool delete_input)
:
	impl(new CL_InputSource_Zipped_Generic)
{
	impl->input = input;
	impl->delete_input = delete_input;
}

CL_InputSource_Zipped::~CL_InputSource_Zipped()
{
	if (impl->delete_input) delete impl->input;
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Zipped Attributes:

int CL_InputSource_Zipped::tell() const
{
	return impl->input->tell();
}

int CL_InputSource_Zipped::size() const
{
	return impl->input->size();
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Zipped Operations:

int CL_InputSource_Zipped::read(void *data, int size)
{
	return impl->input->read(data, size);
}

void CL_InputSource_Zipped::open()
{
	impl->input->open();
}

void CL_InputSource_Zipped::close()
{
	impl->input->close();
}

CL_InputSource *CL_InputSource_Zipped::clone() const
{
	return new CL_InputSource_Zipped(impl->input->clone(), true);
}

void CL_InputSource_Zipped::seek(int pos, SeekEnum seek_type)
{
	impl->input->seek(pos, seek_type);
}

void CL_InputSource_Zipped::push_position()
{
	impl->input->push_position();
}

void CL_InputSource_Zipped::pop_position()
{
	impl->input->pop_position();
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Zipped Implementation:
