/*  $Id: outputsource_memory_generic.h,v 1.7 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_outputsource_memory_generic
#define header_outputsource_memory_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/IOData/outputsource.h"

//: Outputsource writing to memory_generic.
class CL_OutputSource_MemoryGeneric : public CL_OutputSource
{
public:
	CL_OutputSource_MemoryGeneric(unsigned int blocksize = 4*1024);
	virtual ~CL_OutputSource_MemoryGeneric();
	
	//: Get the pointer to the internal data array in the memory_generic source.
	//: Used to extract the data after usage.
	//return: Pointer to the data written.
	void *get_data() const;

	//: Writes larger amounts of data (no endian and 64 bit conversion):
	//param data: Points to the array written.
	//param size: Number of bytes to write.
	//return: Num bytes actually written.
	virtual int write(const void *data, int size);
	
	//: Opens the output source. By default, it is open.
	virtual void open();
	
	//: Closes the output source.
	virtual void close();

	//: Make a copy of the current OutputSource, standing at the same position.
	//return: The copy of the output source.
	virtual CL_OutputSource *clone();

	//: Returns current position in output source.
	//return: Current position in source.
	virtual int tell() const;

	//: Returns the size of the output source
	//return: Current size of output source.
	virtual int size() const;

	//: Cleans out the databuffer without deleting it.
	virtual void purge();

private:
	unsigned int m_blocksize;
	unsigned char *m_data;
	unsigned int m_size, m_pos;
};

#endif
