/*  $Id: directory_scanner_unix.h,v 1.11 2004/01/07 18:36:53 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_directory_scanner_unix
#define header_directory_scanner_unix

#if _MSC_VER > 1000
#pragma once
#endif

#include "../Generic/directory_scanner_generic.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>

class CL_DirectoryScanner_Unix : public CL_DirectoryScanner_Generic
{
//!Construction:
public:
	CL_DirectoryScanner_Unix();
	~CL_DirectoryScanner_Unix();
	
	//: Construct directory scanner for the specified scan expression.
	//: The scan expression only supports * and ? wildcards. Eg: "gfx/*.jpg".
//	CL_DirectoryScanner_Generic(const std::string &scan_exp);

	//: Copy constructor.
//	CL_DirectoryScanner_Generic(const CL_DirectoryScanner &copy);

//!Attributes:
public:
	bool scan (const std::string& pathname);
	bool scan (const std::string& pathname, const std::string& pattern);

	//: Returns the path of the directory being scanned.
	std::string get_directory_path();

	//: Returns the size of the current found file.
	int get_size();

	//: Returns the name of the current found file.
	std::string get_name();
	
	//: Returns the name of the current found file, including the directory path.
	std::string get_pathname();
	
	//: Returns true if filename is a directory.
	bool is_directory();

	//: Returns true if filename is hidden.
	bool is_hidden();

  	//: Returns true if file is readable by current user.
	virtual bool is_readable();

	//: Returns true if file is writable by current user.
	virtual bool is_writable();
	// todo: add other attributes of a file.

//!Operations:
public:
	//: Copy assignment operator.
	//CL_DirectoryScanner_Generic &operator =(const CL_DirectoryScanner_Generic &copy);

	//: Find next file in directory scan. Returns false if no more files was found.
	bool next();
private:
	DIR *dir_temp;
	dirent *entry;
	struct stat statbuf;

	bool use_pattern;
	//Path stored without the trailing slash
	std::string file_name;
	std::string path_name;
	std::string file_pattern;

};

#endif
