/*  $Id: directory_scanner_win32.cpp,v 1.14 2004/01/07 18:36:53 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include "API/Core/System/cl_assert.h"
#include "directory_scanner_win32.h"

/////////////////////////////////////////////////////////////////////////////
// CL_DirectoryScanner_Win32 construction:

CL_DirectoryScanner_Win32::CL_DirectoryScanner_Win32() 
: handle(INVALID_HANDLE_VALUE), first_next(true)
{
}

CL_DirectoryScanner_Win32::CL_DirectoryScanner_Win32(const CL_DirectoryScanner_Win32 &copy) 
: handle(INVALID_HANDLE_VALUE), first_next(true)
{
}

CL_DirectoryScanner_Win32::~CL_DirectoryScanner_Win32()
{
	if (INVALID_HANDLE_VALUE) FindClose(handle);
}

/////////////////////////////////////////////////////////////////////////////
// CL_DirectoryScanner_Win32 attributes:

bool CL_DirectoryScanner_Win32::scan (const std::string &pathname)
{
	return scan(pathname, "*.*");
}

bool CL_DirectoryScanner_Win32::scan (const std::string &pathname, const std::string& pattern)
{
	if (handle != INVALID_HANDLE_VALUE)
	{
		FindClose(handle);
		handle = INVALID_HANDLE_VALUE;
	}
/*
	// Find the full path of the directory we are about to search:
	int size_buffer = GetFullPathName(pathname, 0, 0, 0);
	LPTSTR buffer = new TCHAR[size_buffer];
	LPTSTR filepart = 0;
	GetFullPathName(pathname, size_buffer, buffer, &filepart);
	full_pathname = path_with_ending_slash(buffer);
	delete[] buffer;
*/
	directory_path = path_with_ending_slash(pathname);

	// Start our search:
	std::string filename = directory_path + pattern;
	handle = FindFirstFile(filename.c_str (), &fileinfo);
	first_next = true;
	return (handle != INVALID_HANDLE_VALUE);
}

std::string CL_DirectoryScanner_Win32::get_directory_path()
{
	return directory_path;
}

int CL_DirectoryScanner_Win32::get_size()
{
	return fileinfo.nFileSizeLow;
}

std::string CL_DirectoryScanner_Win32::get_name()
{
	if (first_next) return std::string();
	return std::string(fileinfo.cFileName);
}
	
std::string CL_DirectoryScanner_Win32::get_pathname()
{
	if (first_next) return std::string();
	return directory_path + std::string(fileinfo.cFileName);
}
	
bool CL_DirectoryScanner_Win32::is_directory()
{
	if (first_next) return false;
	return (fileinfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY;
}

bool CL_DirectoryScanner_Win32::is_hidden()
{
	if (first_next) return false;
	return (fileinfo.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) == FILE_ATTRIBUTE_HIDDEN;
}

bool CL_DirectoryScanner_Win32::is_readable()
{
	if (first_next) return false;

	HANDLE file = CreateFile(
		get_pathname().c_str(),
		GENERIC_READ,
		FILE_SHARE_READ,
		0,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING,
		0);

	if (file != INVALID_HANDLE_VALUE) CloseHandle(file);
	return file != INVALID_HANDLE_VALUE;
}

bool CL_DirectoryScanner_Win32::is_writable()
{
	if (first_next) return false;

	HANDLE file = CreateFile(
		get_pathname().c_str(),
		GENERIC_READ,
		FILE_SHARE_READ,
		0,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING,
		0);

	if (file != INVALID_HANDLE_VALUE) CloseHandle(file);
	return file != INVALID_HANDLE_VALUE;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DirectoryScanner_Win32 operations:

bool CL_DirectoryScanner_Win32::next() 
{
	if (handle == INVALID_HANDLE_VALUE) return false;
	if (first_next)
	{
		first_next = false;
		return true;
	}

	return FindNextFile(handle, &fileinfo) == TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DirectoryScanner_Win32 implementation:

std::string CL_DirectoryScanner_Win32::path_with_ending_slash(const std::string &path)
{
	int len = path.length();
	if (len == 0) return ".\\";
	if (path[len-1] == '/' || path[len-1] == '\\') return path;
	return path + "\\";
}
