/*  $Id: inputsource_zip_fileentry.h,v 1.4 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_inputsource_zip_fileentry
#define header_inputsource_zip_fileentry

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/zip_file_entry.h"
#include "zip_local_file_header.h"
#include <stack>
#include <zlib.h>

class CL_InputSource_Zip_FileEntry : public CL_InputSource
{
//! Construction:
public:
	CL_InputSource_Zip_FileEntry(const std::string &filename, const CL_Zip_FileEntry &entry);

	CL_InputSource_Zip_FileEntry(const CL_InputSource_Zip_FileEntry &copy);

	virtual ~CL_InputSource_Zip_FileEntry();

//! Attributes:
public:
	virtual int tell() const;

	virtual int size() const;

//! Operations:
public:
	virtual int read(void *data, int size);

	virtual void open();

	virtual void close();

	virtual CL_InputSource *clone() const;

	virtual void seek(int pos, SeekEnum seek_type);

	virtual void push_position();

	virtual void pop_position();

//! Implementation:
private:
	CL_InputSource *inputsource;

	CL_Zip_FileEntry file_entry;

	CL_Zip_LocalFileHeader file_header;

	cl_int64 pos, compressed_pos;

	z_stream zs;

	std::stack<cl_int64> position_stack;

	bool source_open;

	char zbuffer[16*1024];
};

#endif
