/*  $Id: zip_64_end_of_central_directory_record.cpp,v 1.4 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Core/precomp.h"
#include "zip_64_end_of_central_directory_record.h"
#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/outputsource.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_64EndOfCentralDirectoryRecord construction:

CL_Zip_64EndOfCentralDirectoryRecord::CL_Zip_64EndOfCentralDirectoryRecord()
{
	signature = 0x06064b50;
}
	
CL_Zip_64EndOfCentralDirectoryRecord::~CL_Zip_64EndOfCentralDirectoryRecord()
{
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_Zip_64EndOfCentralDirectoryRecord attributes:


/////////////////////////////////////////////////////////////////////////////
// CL_Zip_64EndOfCentralDirectoryRecord operations:

void CL_Zip_64EndOfCentralDirectoryRecord::load(CL_InputSource *input)
{
	signature = input->read_int32();
	if (signature != 0x06064b50)
	{
		throw CL_Error("Incorrect Zip64 End of Central Directory Record signature");
	}
	
	size_of_record = input->read_int64();
	version_made_by = input->read_int16();
	version_needed_to_extract = input->read_int16();
	number_of_this_disk = input->read_int32();
	number_of_disk_with_central_directory_start = input->read_int32();
	number_of_entries_on_this_disk = input->read_int64();
	number_of_entries_in_central_directory = input->read_int64();
	size_of_central_directory = input->read_int64();
	offset_to_start_of_central_directory = input->read_int64();
	
	// todo: read extensible data sector
}
	
void CL_Zip_64EndOfCentralDirectoryRecord::save(CL_OutputSource *output)
{
	output->write_int32(signature);
	output->write_int64(size_of_record);
	output->write_int16(version_made_by);
	output->write_int16(version_needed_to_extract);
	output->write_int32(number_of_this_disk);
	output->write_int32(number_of_disk_with_central_directory_start);
	output->write_int64(number_of_entries_on_this_disk);
	output->write_int64(number_of_entries_in_central_directory);
	output->write_int64(size_of_central_directory);
	output->write_int64(offset_to_start_of_central_directory);
	output->write(extensible_data_sector.data(), extensible_data_sector.size());
}

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_64EndOfCentralDirectoryRecord implementation:
