/*  $Id: zip_end_of_central_directory_record.cpp,v 1.4 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Core/precomp.h"
#include "zip_end_of_central_directory_record.h"
#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/outputsource.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_EndOfCentralDirectoryRecord construction:

CL_Zip_EndOfCentralDirectoryRecord::CL_Zip_EndOfCentralDirectoryRecord()
{
	signature = 0x06054b50;
}
	
CL_Zip_EndOfCentralDirectoryRecord::~CL_Zip_EndOfCentralDirectoryRecord()
{
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_Zip_EndOfCentralDirectoryRecord attributes:

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_EndOfCentralDirectoryRecord operations:

void CL_Zip_EndOfCentralDirectoryRecord::load(CL_InputSource *input)
{
	signature = input->read_int32();
	if (signature != 0x06054b50)
	{
		throw CL_Error("Incorrect End of Central Directory Record signature");
	}
	
	number_of_this_disk = input->read_int16();
	number_of_disk_with_start_of_central_directory = input->read_int16();
	number_of_entries_on_this_disk = input->read_int16();
	number_of_entries_in_central_directory = input->read_int16();
	size_of_central_directory = input->read_int32();
	offset_to_start_of_central_directory = input->read_int32();
	file_comment_length = input->read_int16();

	char *str = new char[file_comment_length];
	try
	{
		input->read(str, file_comment_length);
		file_comment = std::string(str, file_comment_length);

		delete[] str;
	}
	catch (...)
	{
		delete[] str;
		throw;
	}
}
	
void CL_Zip_EndOfCentralDirectoryRecord::save(CL_OutputSource *output)
{
	output->write_int32(signature);
	output->write_int16(number_of_this_disk);
	output->write_int16(number_of_disk_with_start_of_central_directory);
	output->write_int16(number_of_entries_on_this_disk);
	output->write_int16(number_of_entries_in_central_directory);
	output->write_int32(size_of_central_directory);
	output->write_int32(offset_to_start_of_central_directory);
	output->write_int16(file_comment_length);
	output->write(file_comment.data(), file_comment.length());
}

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_EndOfCentralDirectoryRecord implementation:
