/*  $Id: zip_file_header.cpp,v 1.4 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Core/precomp.h"
#include "zip_file_header.h"
#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/outputsource.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_FileHeader construction:

CL_Zip_FileHeader::CL_Zip_FileHeader()
{
	signature = 0x02014b50;
}
	
CL_Zip_FileHeader::~CL_Zip_FileHeader()
{
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_Zip_FileHeader attributes:


/////////////////////////////////////////////////////////////////////////////
// CL_Zip_FileHeader operations:

void CL_Zip_FileHeader::load(CL_InputSource *input)
{
	signature = input->read_int32();
	if (signature != 0x02014b50)
	{
		throw CL_Error("Incorrect File Header signature");
	}
	version_made_by = input->read_int16();
	version_needed_to_extract = input->read_int16();
	general_purpose_bit_flag = input->read_int16();
	compression_method = input->read_int16();
	last_mod_file_time = input->read_int16();
	last_mod_file_date = input->read_int16();
	crc32 = input->read_int32();
	compressed_size = input->read_int32();
	uncompressed_size = input->read_int32();
	file_name_length = input->read_int16();
	extra_field_length = input->read_int16();
	file_comment_length = input->read_int16();
	disk_number_start = input->read_int16();
	internal_file_attributes = input->read_int16();
	external_file_attributes = input->read_int32();
	relative_offset_of_local_header = input->read_int32();
	filename.resize(file_name_length);

	char *str1 = new char[file_name_length];
	char *str2 = new char[extra_field_length];
	char *str3 = new char[file_comment_length];
	try
	{
		input->read(str1, file_name_length);
		input->read(str2, extra_field_length);
		input->read(str3, file_comment_length);
		filename = std::string(str1, file_name_length);
		extra_field = std::string(str2, extra_field_length);
		file_comment = std::string(str3, file_comment_length);

		delete[] str1;
		delete[] str2;
		delete[] str3;
	}
	catch (...)
	{
		delete[] str1;
		delete[] str2;
		delete[] str3;
		throw;
	}
}

void CL_Zip_FileHeader::save(CL_OutputSource *output)
{
	output->write_int32(signature);
	output->write_int16(version_made_by);
	output->write_int16(version_needed_to_extract);
	output->write_int16(general_purpose_bit_flag);
	output->write_int16(compression_method);
	output->write_int16(last_mod_file_time);
	output->write_int16(last_mod_file_date);
	output->write_int32(crc32);
	output->write_int32(compressed_size);
	output->write_int32(uncompressed_size);
	output->write_int16(file_name_length);
	output->write_int16(extra_field_length);
	output->write_int16(file_comment_length);
	output->write_int16(disk_number_start);
	output->write_int16(internal_file_attributes);
	output->write_int32(external_file_attributes);
	output->write_int32(relative_offset_of_local_header);
	output->write(filename.data(), file_name_length);
	output->write(extra_field.data(), extra_field_length);
	output->write(file_comment.data(), file_comment_length);
}

/////////////////////////////////////////////////////////////////////////////
// CL_Zip_FileHeader implementation:
