/*  $Id: origin.cpp,v 1.4 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"

#include "API/Core/Math/origin.h"

CL_Point calc_origin(CL_Origin origin, const CL_Size &size)
{
	switch(origin)
	{
	case origin_top_left:
	default:
		return CL_Point(0, 0);
		break;
	case origin_top_center:
		return CL_Point(size.width / 2, 0);
		break;
	case origin_top_right:
		return CL_Point(size.width, 0);
		break;
	case origin_center_left:
		return CL_Point(0, size.height / 2);
		break;
	case origin_center:
		return CL_Point(size.width / 2, size.height / 2);
		break;
	case origin_center_right:
		return CL_Point(size.width, size.height / 2);
		break;
	case origin_bottom_left:
		return CL_Point(0, size.height);
		break;
	case origin_bottom_center:
		return CL_Point(size.width / 2, size.height);
		break;
	case origin_bottom_right:
		return CL_Point(size.width, size.height);
		break;
	}
}

CL_Pointf calc_origin(CL_Origin origin, const CL_Sizef &size)
{
	switch(origin)
	{
	case origin_top_left:
	default:
		return CL_Pointf(0, 0);
		break;
	case origin_top_center:
		return CL_Pointf(size.width / 2, 0);
		break;
	case origin_top_right:
		return CL_Pointf(size.width, 0);
		break;
	case origin_center_left:
		return CL_Pointf(0, size.height / 2);
		break;
	case origin_center:
		return CL_Pointf(size.width / 2, size.height / 2);
		break;
	case origin_center_right:
		return CL_Pointf(size.width, size.height / 2);
		break;
	case origin_bottom_left:
		return CL_Pointf(0, size.height);
		break;
	case origin_bottom_center:
		return CL_Pointf(size.width / 2, size.height);
		break;
	case origin_bottom_right:
		return CL_Pointf(size.width, size.height);
		break;
	}
}
