/*  $Id: rect.cpp,v 1.11 2004/02/16 16:24:58 harry Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include "API/Core/Math/rect.h"

#include <cmath>

#define cl_min(a,b) ((a < b) ? a : b)
#define cl_max(a,b) ((a > b) ? a : b)

CL_Rect CL_Rect::get_rot_bounds(const CL_Point &hotspot, float angle) const
{
	//Find the rotated positions of each corner
	CL_Rect retVal(*this);
	CL_Point ul = CL_Point(retVal.left, retVal.top).rotate(hotspot, angle);
	CL_Point ur = CL_Point(retVal.right, retVal.top).rotate(hotspot, angle);
	CL_Point ll = CL_Point(retVal.left, retVal.bottom).rotate(hotspot, angle);
	CL_Point lr = CL_Point(retVal.right, retVal.bottom).rotate(hotspot, angle);
	
	//Use the sidemost corners as the bounds of the new rectangle
	retVal.left = cl_min(cl_min(ul.x, ur.x), cl_min(ll.x, lr.x));
	retVal.right = cl_max(cl_max(ul.x, ur.x), cl_max(ll.x, lr.x));
	retVal.top = cl_min(cl_min(ul.y, ur.y), cl_min(ll.y, lr.y));
	retVal.bottom = cl_max(cl_max(ul.y, ur.y), cl_max(ll.y, lr.y));
	
	return retVal;
}

CL_Rect CL_Rect::get_rot_bounds(CL_Origin origin, int x, int y, float angle) const
{
	return get_rot_bounds(
		CL_Point(left, top) + calc_origin(origin, get_size()) + CL_Point(x, y),
		angle);
}

CL_Rectf CL_Rectf::get_rot_bounds(const CL_Pointf &hotspot, float angle) const
{
	//Find the rotated positions of each corner
	CL_Rectf retVal(*this);
	CL_Pointf ul = CL_Pointf(retVal.left, retVal.top).rotate(hotspot, angle);
	CL_Pointf ur = CL_Pointf(retVal.right, retVal.top).rotate(hotspot, angle);
	CL_Pointf ll = CL_Pointf(retVal.left, retVal.bottom).rotate(hotspot, angle);
	CL_Pointf lr = CL_Pointf(retVal.right, retVal.bottom).rotate(hotspot, angle);
	
	//Use the sidemost corners as the bounds of the new rectangle
	retVal.left = cl_min(cl_min(ul.x, ur.x), cl_min(ll.x, lr.x));
	retVal.right = cl_max(cl_max(ul.x, ur.x), cl_max(ll.x, lr.x));
	retVal.top = cl_min(cl_min(ul.y, ur.y), cl_min(ll.y, lr.y));
	retVal.bottom = cl_max(cl_max(ul.y, ur.y), cl_max(ll.y, lr.y));
	
	return retVal;
}

CL_Rectf CL_Rectf::get_rot_bounds(CL_Origin origin, float x, float y, float angle) const
{
	return get_rot_bounds(
		CL_Pointf(left, top) + calc_origin(origin, get_size()) + CL_Pointf(x, y),
		angle);
}
