/*  $Id: resource_manager_generic.h,v 1.12 2004/02/01 01:54:16 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_resource_manager_generic
#define header_resource_manager_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/Resources/resource.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/Core/XML/dom_document.h"
#include "API/Core/System/weakptr.h"
#include <string>
#include <list>
#include <map>

class CL_InputSourceProvider;
class CL_DomElement;

class CL_ResourceManager_Generic
{
//! Construction:
public:
	CL_ResourceManager_Generic();

	~CL_ResourceManager_Generic();

//! Attributes:
public:
	bool exists(const std::string &res_id);
	
	CL_Resource &get_resource(const std::string &res_id);

	std::list<std::string> get_all_resources();

	std::list<std::string> get_all_resources(const std::string &section_name);

	std::list<std::string> get_all_sections();

	std::list<std::string> get_resources_of_type(const std::string &type_id);

	std::list<std::string> get_resources_of_type(const std::string &type_id, const std::string &section_name);

	CL_InputSourceProvider *get_resource_provider() const { return provider; }

	CL_WeakPtr<CL_ResourceManager_Generic> self;

//! Signals:
public:
	static CL_Signal_v1<CL_Resource &> sig_resource_added;

//! Operations:
public:
	void load(
		const std::string &config_file,
		CL_InputSourceProvider *provider,
		bool delete_inputsource_provider);

	void add_resources(const CL_ResourceManager &additional_resources);

	void remove_resources(const CL_ResourceManager &additional_resources);

	void load_all();

	void unload_all();

	void load_section(const std::string &section_name);

	void unload_section(const std::string &section_name);

	void parse_section(CL_DomElement &sectionElement, const std::string &prefix = std::string());

	void add_sections(std::list<std::string> &returnList, CL_DomElement &sectionElement, const std::string &prefix = std::string());

//! Implementation:
private:
	std::string add_trailing_slash(const std::string &string);

	CL_DomDocument document;
	
	std::map<std::string, CL_Resource> resources;
	
	std::list<CL_ResourceManager> additional_resources;

	CL_InputSourceProvider *provider;

	bool delete_provider;
};

#endif
