/*  $Id: event_listener_generic.h,v 1.12 2004/02/01 03:07:16 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_event_listener_generic
#define header_event_listener_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/System/event_listener.h"
#include "API/Core/System/event_trigger.h"
#include "API/Core/System/sharedptr.h"

class CL_EventTrigger_Generic;

class CL_EventListener_Generic
{
//! Construction:
public:
	CL_EventListener_Generic(CL_EventListener *owner);

	virtual ~CL_EventListener_Generic();

//! Attributes:
public:

//! Operations:
public:
	//: Adds an event trigger to the listener.
	void add_trigger(const CL_EventTrigger *trigger);

	//: Removes an event trigger from the listener.
	void remove_trigger(const CL_EventTrigger *trigger);

	//: Removes all event triggers from the listener.
	void clear();

	//: Resets all event triggers added to the listener.
	void reset_all();

	//: Wait until one of the events triggers. The timeout is in milliseconds.
	//return: true if a event triggered, false if it timed out.
	virtual bool wait(int timeout)=0;

	std::list< CL_SharedPtr<CL_EventTrigger_Generic> > triggers;

//! Implementation:
private:
	CL_EventListener *owner;
};

#endif
