/*  $Id: event_trigger.cpp,v 1.8 2004/02/01 03:07:16 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include "API/Core/System/event_trigger.h"
#include "event_trigger_generic.h"

#ifdef WIN32
#include "Core/System/Win32/event_trigger_win32.h"
#else
#include "Core/System/Unix/event_trigger_unix.h"
#endif

/////////////////////////////////////////////////////////////////////////////
// CL_EventTrigger construction:

CL_EventTrigger::CL_EventTrigger()
{
#ifdef WIN32
	impl = new CL_EventTrigger_Win32;
#else
	impl = new CL_EventTrigger_Unix;
#endif

	reset();
}

CL_EventTrigger::CL_EventTrigger(const CL_EventTrigger &copy)
: impl(copy.impl)
{
}

CL_EventTrigger::~CL_EventTrigger()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_EventTrigger attributes:

bool CL_EventTrigger::get_flag() const
{
	return impl->get_flag();
}

/////////////////////////////////////////////////////////////////////////////
// CL_EventTrigger operations:

void CL_EventTrigger::reset() const
{
	impl->reset();
}

void CL_EventTrigger::set_flag()
{
	impl->set_flag();
}

bool CL_EventTrigger::wait(int timeout)
{
	return impl->wait(timeout);
}

/////////////////////////////////////////////////////////////////////////////
// CL_EventTrigger implementation:
