
#include "Core/precomp.h"
#include "API/Core/XML/xml_token_load.h"
#include "xml_token_load_generic.h"
#include "API/Core/XML/xml_token_string.h"

/////////////////////////////////////////////////////////////////////////////
// CL_XMLTokenLoad construction:

CL_XMLTokenLoad::CL_XMLTokenLoad()
	: impl(new CL_XMLTokenLoad_Generic)
{
}
	
CL_XMLTokenLoad::CL_XMLTokenLoad(const CL_XMLTokenLoad &copy)
	: impl(copy.impl)
{
}
	
CL_XMLTokenLoad::~CL_XMLTokenLoad()
{
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_XMLTokenLoad attributes:

CL_XMLToken::TokenType CL_XMLTokenLoad::get_type() const
{
	return impl->type;
}

CL_XMLToken::TokenVariant CL_XMLTokenLoad::get_variant() const
{
	return impl->variant;
}

std::string CL_XMLTokenLoad::get_name() const
{
	return impl->name.to_string();
}

std::string CL_XMLTokenLoad::get_value() const
{
	return impl->value.to_string();
}

int CL_XMLTokenLoad::get_attributes_number() const
{
	return impl->attributes.size();
}

std::pair<std::string, std::string> CL_XMLTokenLoad::get_attribute(int attribute_num) const
{
	if (attribute_num < 0 || attribute_num >= (int)impl->attributes.size())
		return std::make_pair(std::string(), std::string());

	std::pair<CL_XMLTokenString, CL_XMLTokenString> const & attribute = impl->attributes[attribute_num];
	return std::make_pair(attribute.first.to_string(), attribute.second.to_string());
}

const std::pair<CL_XMLTokenString, CL_XMLTokenString> & CL_XMLTokenLoad::get_attribute_fast(int attribute_num) const
{
	return impl->attributes[attribute_num];
}

/////////////////////////////////////////////////////////////////////////////
// CL_XMLTokenLoad operations:

void CL_XMLTokenLoad::set_type(TokenType type)
{
	impl->type = type;
}

void CL_XMLTokenLoad::set_variant(TokenVariant variant)
{
	impl->variant = variant;
}

void CL_XMLTokenLoad::set_name(const CL_XMLTokenString & name)
{
	impl->name = name;
}

void CL_XMLTokenLoad::set_value(const CL_XMLTokenString & value)
{
	impl->value = value;
}

void CL_XMLTokenLoad::set_attribute(const CL_XMLTokenString & name, const CL_XMLTokenString & value)
{
	int size = impl->attributes.size();
	for (int i=0; i<size; i++)
	{
		if (impl->attributes[i].first == name)
		{
			impl->attributes[i].second = value;
			return;
		}
	}
	impl->attributes.push_back(std::make_pair(name, value));
}

/////////////////////////////////////////////////////////////////////////////
// CL_XMLTokenLoad implementation:
