/*  $Id: jpeg_provider_generic.h,v 1.9 2004/01/24 00:54:53 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_jpeg_provider_generic
#define header_jpeg_provider_generic

#if _MSC_VER > 1000
#pragma once
#endif

#ifdef WIN32
#define XMD_H
#endif

#include <stdio.h>
extern "C"
{
	#undef FAR
	#include <jpeglib.h>
}

#include "API/Display/Providers/jpeg_provider.h"
#include "Display/pixel_buffer_generic.h"

#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/inputsource_provider.h"

class CL_InputSourceProvider;

//: Surface provider that can load JPEG (.jpg) files.
class CL_JPEGProvider_Generic : public CL_PixelBuffer_Generic
{
//! Construction:
public:
	//: Constructs a surface provider that can read JPEG files.
	CL_JPEGProvider_Generic(
		const std::string &name, 
		CL_InputSourceProvider *provider);

	~CL_JPEGProvider_Generic();

//! Attributes:
public:
	//: Returns the pitch of the image (bytes per line).
	unsigned int get_pitch() const { return pitch; }

	//: Returns the width of the image.
	unsigned int get_width() const { return width; }

	//: Returns the height of the image.
	unsigned int get_height() const { return height; }
	
	//: Returns the number of subsprites in the image.
	unsigned int get_num_frames() const { return 1; }

	//: Returns the pixelformat used by the image.
	// EPixelFormat get_pixel_format() const { return RGB888; }

	//: Returns the red color mask used by the target.
	unsigned int get_red_mask() const;

	//: Returns the green color mask by the target.
	unsigned int get_green_mask() const;

	//: Returns the blue color mask by the target.
	unsigned int get_blue_mask() const;

	//: Returns the alpha mask by the target.
	unsigned int get_alpha_mask() const;

//! Operations:
public:
	//: Returns the image data. Provider must be locked before pointer is valid.
	void *get_data();

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock() { return; }

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock() { return; }

	//: Loads the JPEG data into the pixel buffer.
	void init();

	//: Cleans up.
	void deinit();

//! Implementation:
private:
	// Functions for managing the input handling
	static void jpeg_InputSource_src (j_decompress_ptr cinfo, CL_JPEGProvider_Generic* prov); 

	static void init_source (j_decompress_ptr cinfo);

	static boolean fill_input_buffer (j_decompress_ptr cinfo);

	static void skip_input_data (j_decompress_ptr cinfo, long num_bytes);

	static void term_source (j_decompress_ptr cinfo);

	std::string filename;

	unsigned char *image;

	int pitch;

	int width, height;

	int bpp;

	int pos;

	typedef struct
	{
		struct jpeg_source_mgr   pub;           /* public fields */
		CL_JPEGProvider_Generic* jpeg_provider; /* source stream */
		JOCTET *                 buffer;        /* start of buffer */
		boolean                  start_of_file; /* have we gotten any data yet? */
	} InputSource_source_mgr;
	
	typedef InputSource_source_mgr* InputSource_src_ptr;

	CL_InputSourceProvider* input_provider;

	CL_InputSource* input_source;
};

#endif
