/*  $Id: pcx_provider_generic.h,v 1.12 2004/01/24 21:52:07 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_pcx_provider_generic
#define header_pcx_provider_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Display/Providers/pcx_provider.h"
#include "Display/pixel_buffer_generic.h"
#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/inputsource_provider.h"

//: Surface provider that can load PCX files.
class CL_PCXProvider_Generic : public CL_PixelBuffer_Generic
{
//! Construction:
public:
	//: Constructs a surface provider that can read pcx files.
	//param name: Name of the pcx file to load.
	//param provider: Input source provider that delivers the pcx file.
	CL_PCXProvider_Generic(
		std::string name,
		CL_InputSourceProvider *provider);

	//: PCX Provider destructor
	virtual ~CL_PCXProvider_Generic();

//! Operations:
public:
	//: Returns the image data. Provider must be locked before pointer is valid.
	virtual void *get_data() { return image; }

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock() { return; }

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock() { return; }

//! Implementation:
private:
	//: Loads the pcx
	void read_pcx(
		std::string _name, 
		CL_InputSource *_datafile);

	//: Image data.
	unsigned char *image;
};

#endif
