/*  $Id: targa_provider_generic.h,v 1.12 2004/01/27 23:10:40 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_targaprovider_generic
#define header_targaprovider_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Display/Providers/targa_provider.h"
#include "Display/pixel_buffer_generic.h"

#include "API/Core/IOData/inputsource.h"
#include "API/Core/IOData/inputsource_provider.h"

//: Surface provider that can load targa (.tga) files.
class CL_TargaProvider_Generic : public CL_PixelBuffer_Generic
{
//! Construction:
public:
	//: Constructs a surface provider that can read targa files.
	//param name: Name of the targa file to load.
	//param provider: Input source provider that delivers the targa file.
	CL_TargaProvider_Generic(
		const std::string name,
		CL_InputSourceProvider *provider);

	//: Target Provider Destructor
	virtual ~CL_TargaProvider_Generic();

//! Operations:
public:
	//: Returns the image data. Provider must be locked before pointer is valid.
	virtual void *get_data();

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock() { return; }

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock() { return; }

//! Implementation:
private:
	struct TGA_Header;

	unsigned char *image;
							
	void read_tga(CL_InputSource *input_source);

	void read_colormapped(CL_InputSource *input_source, const TGA_Header &header);
	void read_rgb(CL_InputSource *input_source, const TGA_Header &header);
	void read_rle_colormapped(CL_InputSource *input_source, const TGA_Header &header);
	void read_rle_rgb(CL_InputSource *input_source, const TGA_Header &header);
};

#endif
