/*  $Id: display_mode.cpp,v 1.12 2003/11/11 10:13:24 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef WIN32
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#define V_DBLSCAN       0x020
#else
#include <windows.h>
#endif

#include "Display/display_precomp.h"
#include "API/Display/display_mode.h"
#include "API/Core/System/clanstring.h"
#include "display_mode_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayMode construction:

CL_DisplayMode::CL_DisplayMode(const CL_Size &res, int bpp, int refresh_rate) : impl(0)
{
	impl = new CL_DisplayMode_Generic;
	impl->resolution = res;
	impl->bpp = bpp;
	impl->refresh_rate = refresh_rate;
}

CL_DisplayMode::CL_DisplayMode(const CL_DisplayMode &copy) : impl (new CL_DisplayMode_Generic)
{
	*impl = *copy.impl;
}

CL_DisplayMode::~CL_DisplayMode()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayMode attributes:

CL_Size CL_DisplayMode::get_resolution() const
{
	return impl->resolution;
}

int CL_DisplayMode::get_bpp() const
{
	return impl->bpp;
}

int CL_DisplayMode::get_refresh_rate() const
{
	return impl->refresh_rate;
}

std::string CL_DisplayMode::get_string() const
{
	return CL_String::format("%1x%2x%3@%4", impl->resolution.width, impl->resolution.height, impl->bpp, impl->refresh_rate);
}

std::vector<CL_DisplayMode> &CL_DisplayMode::get_display_modes()
{
	static std::vector<CL_DisplayMode> modes;

	if(modes.empty())
	{
#ifdef WIN32
		int index = 0;
		while (true)
		{
			DEVMODE cur_devmode;
			memset(&cur_devmode, 0, sizeof(DEVMODE));
			cur_devmode.dmSize = sizeof(DEVMODE);
			if (EnumDisplaySettings(0, index++, &cur_devmode) == 0) break;

			modes.push_back(CL_DisplayMode(
				CL_Size(cur_devmode.dmPelsWidth, cur_devmode.dmPelsHeight),
				cur_devmode.dmBitsPerPel,
				cur_devmode.dmDisplayFrequency));
		}
#else
		Display *display = XOpenDisplay(NULL);

		int event_base;
		int error_base;

		// Check for VidMode extension
		if (XF86VidModeQueryExtension(display, &event_base, &error_base))
		{
			int screen = DefaultScreen(display);

			XF86VidModeModeInfo **vmodes;
			int nmodes;

			if (XF86VidModeGetAllModeLines(display, screen, &nmodes, &vmodes))
			{
				int bpp = DefaultDepth(display, screen);
				int refresh_rate;

				for (int i = 0; i < nmodes; i++)
				{
					if ((vmodes[i]->flags) & V_DBLSCAN)
						refresh_rate = vmodes[i]->dotclock * 500 / (vmodes[i]->htotal * vmodes[i]->vtotal);
					else
						refresh_rate = vmodes[i]->dotclock * 1000 / (vmodes[i]->htotal * vmodes[i]->vtotal);

					modes.push_back(CL_DisplayMode(
						CL_Size(vmodes[i]->hdisplay, vmodes[i]->vdisplay),
						bpp,
						refresh_rate));
				}
			}
		}

		XCloseDisplay(display);
#endif
	}

	return modes;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayMode attributes:

CL_DisplayMode &CL_DisplayMode::operator =(const CL_DisplayMode &copy)
{
	*impl = *copy.impl;
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayMode implementation:
