/*  $Id: font_target.h,v 1.2 2004/02/25 22:20:05 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_font_target
#define header_font_target

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Display/blend_func.h"
#include "API/Core/Math/origin.h"

class CL_Font_Generic;
class CL_GraphicContext;

class CL_Font_Target
{
//! Construction:
public:
	CL_Font_Target(CL_Font_Generic *attributes) : attributes(attributes) { return; }

	virtual ~CL_Font_Target() { return; }

//! Attributes:
public:
	//: If 0, variable width font. If another number, all characters are this width
	virtual int get_fixed_width() const=0;
	
	//: Gets the width of a single glyph directly from the map, including scaling and offsets
	virtual int get_width(unsigned int chr) const=0;
	
	//: Gets the height of the font, including scaling and offsets
	virtual int get_height() const=0;

	//: Returns true if the given character has any associated glyph
	virtual bool is_glyph(unsigned int chr) const=0;

	virtual void get_scale(float& x, float& y) const=0;

	virtual float get_alpha() const=0;
	
	virtual void get_color(float& r, float& g, float& b, float& a) const=0;
	
	virtual void get_blend_func(CL_BlendFunc &src, CL_BlendFunc &dest) const=0;
	
	virtual void get_rot_hotspot(CL_Origin &origin, int &x, int &y) const=0;

protected:
	CL_Font_Generic *attributes;

//! Operations:
public:
	virtual void set_scale(float x, float y)=0;

	virtual void set_alpha(float a)=0;

	virtual void set_color(float r, float g, float b, float a)=0;

	virtual void set_blend_func(CL_BlendFunc src, CL_BlendFunc dest)=0;

	virtual void set_rot_hotspot(CL_Origin origin, int x, int y)=0;

	//: Draws a glyph at a given location.
	//param x, y: The upper-left coordinates of where to draw the glyph.
	//param chr: The character to draw the glyph of.
	//param ang: The amount to rotate the glyph by. The hotspot is set with CL_Font::set_glyph_rot_hotspot().
	//param gc: Graphic context to render to.
	//- <p> If there is no such glyph for that character, nothing is drawn. </p>
	virtual void draw_glyph(int x, int y, unsigned int chr, float ang, CL_GraphicContext *gc) const=0;

	//: Draws a single character to a given spot.
	//param x, y: The upper-left coordinates of where to draw the character.
	//param chr: The character to draw the glyph of.
	//param ang: The amount to rotate the glyph by. The hotspot is set with CL_Font::set_glyph_rot_hotspot().
	//param gc: Graphic context to render to.
	//- <p> If there is no glyph for the given character, then nothing is drawn. </p>
	virtual void draw_character(int x, int y, unsigned int chr, float ang, CL_GraphicContext *gc) const=0;

//! Implementation:
private:
};

#endif
