/*  $Id: key_binding.cpp,v 1.3 2003/07/20 22:31:49 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "API/Display/key_binding.h"
#include "key_binding_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_KeyBinding construction:

CL_KeyBinding::CL_KeyBinding()
: impl(new CL_KeyBinding_Generic)
{
}

CL_KeyBinding::CL_KeyBinding(int keycode, const CL_InputDevice &device)
: impl(new CL_KeyBinding_Generic)
{
	set_keycode(keycode);
	set_device(device);
}

CL_KeyBinding::CL_KeyBinding(const CL_KeyBinding &copy)
: impl(new CL_KeyBinding_Generic)
{
	set_keycode(copy.get_keycode());
	set_device(copy.get_device());
}

/////////////////////////////////////////////////////////////////////////////
// CL_KeyBinding attributes:

bool CL_KeyBinding::operator ==(const CL_KeyBinding &other) const
{
	// TODO: Check if inputdevices also match
	return other.get_keycode() == get_keycode();
}

bool CL_KeyBinding::operator <(const CL_KeyBinding &other) const
{
	// TODO: Check if inputdevices also match
	return other.get_keycode() < get_keycode();
}

bool CL_KeyBinding::operator >(const CL_KeyBinding &other) const
{
	// TODO: Check if inputdevices also match
	return other.get_keycode() > get_keycode();
}

CL_InputDevice &CL_KeyBinding::get_device() const
{
	return impl->device;
}

int CL_KeyBinding::get_keycode() const
{
	return impl->keycode;
}

std::string CL_KeyBinding::get_key_name() const
{
	return "TODO";
}

/////////////////////////////////////////////////////////////////////////////
// CL_KeyBinding operations:

CL_KeyBinding &CL_KeyBinding::operator =(const CL_KeyBinding &copy)
{
	set_keycode(copy.get_keycode());
	set_device(copy.get_device());
	return *this;
}

void CL_KeyBinding::set_keycode(int keycode)
{
	impl->keycode = keycode;
}

void CL_KeyBinding::set_device(const CL_InputDevice &device)
{
	impl->device = device;
}

/////////////////////////////////////////////////////////////////////////////
// CL_KeyBinding implementation:
