/*  $Id: pixel_buffer_generic.h,v 1.8 2004/01/15 13:23:39 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_pixel_buffer_generic
#define header_pixel_buffer_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Display/pixel_format.h"
#include "API/Display/palette.h"

//: Pixel data implementation interface.
class CL_PixelBuffer_Generic
{
//! Construction:
public:
	CL_PixelBuffer_Generic() : lock_count(0), ref_count(0) { return; }

	virtual ~CL_PixelBuffer_Generic() { return; }

//! Attributes:
public:
	//: Buffer pixel format description.
	CL_PixelFormat format;

	//: Palette, if 8 bpp pixel format.
	CL_Palette palette;

	//: Buffer scanline pitch.
	unsigned int pitch;

	//: Buffer width.
	int width;

	//: Buffer height.
	int height;

	//: Lock reference count.
	int lock_count;

//! Operations:
public:
	//: Returns a pointer to the beginning of the pixel buffer. Pointer
	//: is only valid inside a lock/unlock session.
	virtual void *get_data()=0;

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock()=0;

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock()=0;

	int add_ref() { return ++ref_count; }

	int release_ref()
	{
		ref_count--;
		if (ref_count == 0) { delete this; return 0; }
		return ref_count;
	}
	
	//: Converts between two RGBA formats.
	static void convert(
		void* in_buffer,
		const CL_PixelFormat& in_pf,
		int in_pitch,
		void* out_buffer,
		const CL_PixelFormat& out_pf,
		int out_pitch,
		const CL_Size& size);

	//: Converts from palette format to RGBA.
	static void convert_pal(
		void* in_buffer,
		const CL_PixelFormat& in_pf,
		int in_pitch,
		const CL_Palette& in_pal,
		void* out_buffer,
		const CL_PixelFormat& out_pf,
		int out_pitch,
		const CL_Size& size);

//! Implementation:
private:
	int ref_count;
};

#endif
