/*  $Id: pixel_buffer_memory.h,v 1.8 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_pixel_buffer_memory
#define header_pixel_buffer_memory

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Display/pixel_buffer.h"
#include "pixel_buffer_generic.h"

//: Memory pixel data implementation.
class CL_PixelBuffer_Memory : public CL_PixelBuffer_Generic
{
//! Construction:
public:
	CL_PixelBuffer_Memory(int width, int height, int pitch, const CL_PixelFormat &format, void *data = 0);

	CL_PixelBuffer_Memory(int width, int height, int pitch, const CL_PixelFormat &format, const CL_Palette &palette, void *data = 0);

	virtual ~CL_PixelBuffer_Memory();

//! Operations:
public:
	//: Returns a pointer to the beginning of the pixel buffer. Pointer
	//: is only valid inside a lock/unlock session.
	virtual void *get_data();

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock();

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock();

//! Implementation:
private:
	unsigned char *data;

	bool delete_data;
};

#endif
