/*  $Id: surface_generic.cpp,v 1.13 2004/04/20 13:36:27 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "surface_generic.h"
#include "surface_target.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_Generic construction:

CL_Surface_Generic::CL_Surface_Generic(CL_Surface_Target *target) :
	target(target),
	angle(0.0f),
	angle_pitch(0.0f),
	angle_yaw(0.0f),
	scale_x(1.0f),
	scale_y(1.0f),
	red(1.0f),
	green(1.0f),
	blue(1.0f),
	alpha(1.0f),
	blend_src(blend_src_alpha),
	blend_dest(blend_one_minus_src_alpha),
	translation_hotspot(0,0),
	rotation_hotspot(0,0),
	translation_origin(origin_top_left),
	rotation_origin(origin_center),
	ref_count(0)
{
	if (target) target->add_ref();
}	

CL_Surface_Generic::~CL_Surface_Generic()
{
	if (target) target->release_ref();
}

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_Generic attributes:


/////////////////////////////////////////////////////////////////////////////
// CL_Surface_Generic operations:

CL_Surface_Generic &CL_Surface_Generic::operator =(const CL_Surface_Generic &copy)
{
	if (target) target->release_ref();
	target = copy.target;
	angle = copy.angle;
	angle_pitch = copy.angle_pitch;
	angle_yaw = copy.angle_yaw;
	scale_x = copy.scale_x;
	scale_y = copy.scale_y;
	alpha = copy.alpha;
	blend_src = copy.blend_src;
	blend_dest = copy.blend_dest;
	translation_hotspot = copy.translation_hotspot;
	rotation_hotspot = copy.rotation_hotspot;
	translation_origin = copy.translation_origin;
	rotation_origin = copy.rotation_origin;
	if (target) target->add_ref();

	return *this;
}

CL_Surface_Generic *CL_Surface_Generic::copy_on_write()
{
	if (ref_count == 1) return this;

	CL_Surface_Generic *copy = new CL_Surface_Generic(0);
	*copy = *this;
	copy->add_ref();
	release_ref();
	return copy;
}

void CL_Surface_Generic::release_ref()
{
	ref_count--;
	if (ref_count == 0) delete this;
}

void CL_Surface_Generic::add_ref()
{
	ref_count++;
}

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_Generic implementation:
