/*  $Id: surface_target.h,v 1.20 2004/04/24 00:09:49 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_surface_target
#define header_surface_target

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/Math/origin.h"
#include "API/Display/blend_func.h"

class CL_PixelBuffer;
class CL_Surface_Generic;
class CL_GraphicContext;
class CL_Point;
class CL_Pointf;
class CL_Sizef;
class CL_Rect;
struct CL_Surface_DrawParams1;
struct CL_Surface_DrawParams2;

struct CL_Surface_TargetDrawParams1
{
	double pixDestX;
	double pixDestY;
	double destWidth;
	double destHeight;
	CL_Pointf rotation_hotspot;
};

class CL_Surface_Target
{
//! Construction:
public:
	CL_Surface_Target();

	virtual ~CL_Surface_Target();

//! Attributes:
public:
	//: Returns the width.
	virtual int get_width() const=0;
	
	//: Returns the height.
	virtual int get_height() const=0;

	//: Returns a pixelbuffer object for accessing surface data.
	virtual CL_PixelBuffer get_pixeldata()=0;

//! Operations:
public:
	//: Uploads pixel data to surface.
	//!param pos: Position on surface where pixel data should be put.
	//!param src_rect: Part of pixelbuffer that should be uploaded.
	//!param pixeldata: CL_PixelBuffer containing the image to be uploaded.
	virtual void set_pixeldata(const CL_Point &pos, const CL_Rect &src_rect, CL_PixelBuffer &pixeldata)=0;

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		CL_Surface_DrawParams1 *params1,
		CL_GraphicContext *gc)=0;

	//: Draw surface on screen.
	void draw(
		CL_Surface_Generic *attributes,
		CL_Surface_DrawParams2 *params2,
		CL_GraphicContext *context);

	virtual void draw(
		CL_Surface_Generic *attributes,
		CL_Surface_DrawParams2 *params2,
		CL_Surface_TargetDrawParams1 *target_params1,
		CL_GraphicContext *context);

	void add_ref();

	void release_ref();

	// inlined this function for performance reasons.
	CL_Pointf calc_hotspot(CL_Origin origin, float hotspot_x, float hotspot_y, float size_width, float size_height)
	{
		switch(origin)
		{
		case origin_top_left:
		default:
			return CL_Pointf(-hotspot_x, hotspot_y);
			break;
		case origin_top_center:
			return CL_Pointf(size_width / 2 - hotspot_x, -hotspot_y);
			break;
		case origin_top_right:
			return CL_Pointf(size_width - hotspot_x, -hotspot_y);
			break;
		case origin_center_left:
			return CL_Pointf(-hotspot_x, size_height / 2 - hotspot_y);
			break;
		case origin_center:
			return CL_Pointf(size_width / 2 - hotspot_x, size_height / 2 - hotspot_y);
			break;
		case origin_center_right:
			return CL_Pointf(size_width - hotspot_x, size_height / 2 - hotspot_y);
			break;
		case origin_bottom_left:
			return CL_Pointf(-hotspot_x, size_height - hotspot_y);
			break;
		case origin_bottom_center:
			return CL_Pointf(size_width / 2 - hotspot_x, size_height - hotspot_y);
			break;
		case origin_bottom_right:
			return CL_Pointf(size_width - hotspot_x, size_height - hotspot_y);
			break;
		}
	}

	#define calc_rotate_x(px,py,rotation_hotspot_x,rotation_hotspot_y,rotate_x_x,rotate_y_x) \
		(rotation_hotspot_x + (px-rotation_hotspot_x) * rotate_x_x + (py-rotation_hotspot_y) * rotate_y_x)

	#define calc_rotate_y(px,py,rotation_hotspot_x,rotation_hotspot_y,rotate_x_y,rotate_y_y) \
		(rotation_hotspot_y + (px-rotation_hotspot_x) * rotate_x_y + (py-rotation_hotspot_y) * rotate_y_y)

/*

	double calc_rotate_x(
		const double &px,
		const double &py,
		const double &rotation_hotspot_x,
		const double &rotation_hotspot_y,
		const double &rotate_x_x,
		const double &rotate_y_x)
	{
		return 
			rotation_hotspot_x +
			(px-rotation_hotspot_x) * rotate_x_x +
			(py-rotation_hotspot_y) * rotate_y_x;
	}

	double calc_rotate_y(
		const double &px,
		const double &py,
		const double &rotation_hotspot_x,
		const double &rotation_hotspot_y,
		const double &rotate_x_y,
		const double &rotate_y_y)
	{
		return 
			rotation_hotspot_y +
			(px-rotation_hotspot_x) * rotate_x_y +
			(py-rotation_hotspot_y) * rotate_y_y;
	}
*/
//! Implementation:
private:
	int ref_count;
};

#endif
