/*  $Id: opengl_target.cpp,v 1.10 2004/02/18 14:32:15 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "opengl_target.h"
#ifdef WIN32
// #include "Display/Win32/font_win32.h"
#include "WGL/display_window_opengl.h"
#include "API/Display/Providers/dc_buffer.h"
#include "API/Display/sprite_description.h"
#include "API/Display/pixel_format.h"
#else
// #include "GLX/font_x11.h"
#include "GLX/display_window_opengl.h"
#endif
#include "surface_target_opengl.h"
#include "canvas_opengl.h"
#include "Display/font_target_sprite.h"

/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLTarget construction:


/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLTarget attributes:


/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLTarget operations:

CL_DisplayWindow_Generic *CL_OpenGLTarget::create_window()
{
	return new CL_DisplayWindow_OpenGL;
}

CL_Surface_Target *CL_OpenGLTarget::create_surface(
	CL_PixelBuffer *provider,
	bool delete_provider,
	int flag)
{
	return new CL_Surface_Target_OpenGL(provider, delete_provider, flag);
}

CL_Canvas_Generic *CL_OpenGLTarget::create_canvas()
{
	return new CL_Canvas_OpenGL;
}

CL_Font_Target* CL_OpenGLTarget::create_system_font_target(
	CL_Font_Generic *attributes,
	const std::string &font_name,
	const std::string &letters,
	int font_height,
	int font_width,
	bool bold,
	bool italic,
	bool underline,
	bool strikeout)
{
#ifdef WIN32
	// Create the font we need:
	HFONT font = CreateFont(
		font_height,
		font_width,
		0,
		0,
		bold ? FW_BOLD : FW_NORMAL,
		italic ? TRUE : FALSE,
		underline ? TRUE : FALSE,
		strikeout ? TRUE : FALSE,
		DEFAULT_CHARSET,
		OUT_DEFAULT_PRECIS,
		CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY,
		FF_DONTCARE,
		font_name.c_str());

	const int num_chars = letters.length();
	int i;
	int total_width = 0;
	int pos = 0;
	HDC screen_dc;
	HFONT old_font;
	INT widths[256];

	// Get the glyph widths:
	screen_dc = GetDC(0);
	old_font = (HFONT) SelectObject(screen_dc, font);
	GetCharWidth32(screen_dc, 0, 255, widths);
	SelectObject(screen_dc, old_font);

	// Figure out how big our device context has to be:
	for (i=0; i<num_chars; i++)
	{
		total_width += widths[static_cast<unsigned char>(letters[i])];
	}

	// Create device context, set it up for glyph drawing:
	CL_DeviceContextBuffer dc_buffer(screen_dc, total_width, font_height, true);
	RECT rc = { 0, 0, total_width, font_height };
	SetBkColor(dc_buffer.get_dc(), RGB(0,0,0));
	FillRect(dc_buffer.get_dc(), &rc, (HBRUSH) GetStockObject(BLACK_BRUSH));
	SetTextColor(dc_buffer.get_dc(), RGB(255,255,255));
	SetBkMode(dc_buffer.get_dc(), OPAQUE);
	old_font = (HFONT) SelectObject(dc_buffer.get_dc(), font);

	// Draw each glyph:
	for (i=0; i<num_chars; i++)
	{
		const unsigned char cur = static_cast<unsigned char>(letters[i]);
		TextOut(dc_buffer.get_dc(), pos, 0, (char *)(&cur), 1);
		pos += widths[cur];
	}
	SelectObject(dc_buffer.get_dc(), old_font);
	DeleteObject(font);

	// Add each glyph as a frame:
	CL_SpriteDescription desc;
	pos = 0;
	for (i=0; i<num_chars; i++)
	{
		const int character = static_cast<unsigned char>(letters[i]);
		desc.add_gridclipped_frames(&dc_buffer, pos, 0, widths[character], font_height, 1, 1);
		pos += widths[character];
	}

	// Set up the actual font data:
	return new CL_Font_Target_Sprite(
		attributes,
		CL_Sprite(desc),
		letters);
#else
	return 0;
/*		new CL_Font_X11(
		attributes,
		font_name,
		letters,
		height,
		width,
		bold,
		italic,
		underline,
		strikeout);
*/
#endif
}

bool CL_OpenGLTarget::enable_packer()
{
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLTarget implementation:
