/*  $Id: surface_target_opengl.h,v 1.16 2004/04/23 22:38:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_surface_target_opengl
#define header_surface_target_opengl

#if _MSC_VER > 1000
#pragma once
#endif

#include "Display/surface_generic.h"
#include "Display/surface_target.h"
#include "API/Display/surface.h"
#include "API/Core/Math/size.h"
#include "GL/gl_state_checker.h"

class CL_Canvas_OpenGL;

class CL_Surface_Target_OpenGL : public CL_Surface_Target
{
//! Construction:
public:
	CL_Surface_Target_OpenGL(
		CL_PixelBuffer *provider,
		bool delete_provider,
		int flag);

	CL_Surface_Target_OpenGL(
		const CL_PixelFormat &pf,
		const CL_Size &size);

	virtual ~CL_Surface_Target_OpenGL();

//! Attributes:
public:
	//: Returns the width.
	virtual int get_width() const;
	
	//: Returns the height.
	virtual int get_height() const;

	//: Returns a pixelbuffer object for accessing surface data.
	virtual CL_PixelBuffer get_pixeldata();
	
	//: Returns a graphic context object for rendering to this surface.
	virtual CL_GraphicContext *get_gc();

	//: OpenGL texture handle.
	CLuint handle;

	CL_Size surface_size;

	CL_Size texture_size;

	CL_Canvas_OpenGL *selected_canvas;

	CL_PixelBuffer *original_pixelbuffer;

//! Operations:
public:
	//: Uploads pixel data to surface.
	//!param pos: Position on surface where pixel data should be put.
	//!param pixeldata: CL_PixelBuffer containing the image to be uploaded.
	//!param src_rect: Part of pixelbuffer that should be uploaded.
	virtual void set_pixeldata(const CL_Point &pos, const CL_Rect &src_rect, CL_PixelBuffer &pixeldata);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		CL_Surface_DrawParams1 *params1,
		CL_GraphicContext *gc);

//! Implementation:
private:
	CL_GLState state;
};

#endif
