/*  $Id: component_resize_handler.cpp,v 1.20 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "API/GUI/component_resize_handler.h"
#include "API/GUI/component.h"
#include "API/Display/keys.h"

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentResizeHandler_Generic class:

class CL_ComponentResizeHandler_Generic
{
public:
	CL_ComponentResizeHandler_Generic(CL_ComponentResizeHandler *resize_handler);

	void on_mouse_down(const CL_InputEvent &key);
	void on_mouse_up(const CL_InputEvent &key);
	void on_mouse_move(const CL_InputEvent &key);

	CL_Slot slot_mouse_down;
	CL_Slot slot_mouse_up;
	CL_Slot slot_mouse_move;

	bool is_active;
	CL_Point mouse_origin;

	CL_ComponentResizeHandler *resize_handler;
};

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentResizeHandler:

CL_ComponentResizeHandler::CL_ComponentResizeHandler(CL_Component *owner)
: CL_Component(owner)
{
	impl = new CL_ComponentResizeHandler_Generic(this);
}

CL_ComponentResizeHandler::CL_ComponentResizeHandler(const CL_Rect &resize_area, CL_Component *owner)
: CL_Component(resize_area, owner)
{
	impl = new CL_ComponentResizeHandler_Generic(this);
}

CL_ComponentResizeHandler::~CL_ComponentResizeHandler()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentResizeHandler_Generic implementation:

CL_ComponentResizeHandler_Generic::CL_ComponentResizeHandler_Generic(CL_ComponentResizeHandler *resize_handler)
: resize_handler(resize_handler)
{
	is_active = false;

	slot_mouse_down = resize_handler->sig_mouse_down().connect(
		this, &CL_ComponentResizeHandler_Generic::on_mouse_down);

	slot_mouse_up = resize_handler->sig_mouse_up().connect(
		this, &CL_ComponentResizeHandler_Generic::on_mouse_up);

	slot_mouse_move = resize_handler->sig_mouse_move().connect(
		this, &CL_ComponentResizeHandler_Generic::on_mouse_move);
}

void CL_ComponentResizeHandler_Generic::on_mouse_down(const CL_InputEvent &key)
{
	if(key.id == CL_MOUSE_LEFT)
	{
		mouse_origin = CL_Point((int)key.mouse_pos.x, (int)key.mouse_pos.y);
		is_active = true;
		resize_handler->capture_mouse();
	}
}

void CL_ComponentResizeHandler_Generic::on_mouse_up(const CL_InputEvent &key)
{
	if(key.id == CL_MOUSE_LEFT)
	{
		is_active = false;
		resize_handler->release_mouse();
	}
}

void CL_ComponentResizeHandler_Generic::on_mouse_move(const CL_InputEvent &key)
{
	if(is_active == false)
		return;

	CL_Component *owner = resize_handler->get_parent();
	
	int delta_x = key.mouse_pos.x - mouse_origin.x;
	int delta_y = key.mouse_pos.y - mouse_origin.y;

	owner->set_size(owner->get_width() + delta_x, owner->get_height() + delta_y);
}
