/*  $Id: label.cpp,v 1.53 2003/09/21 20:20:32 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "API/GUI/label.h"
#include "API/GUI/stylemanager.h"
#include "API/Core/XML/dom_element.h"
#include "label_generic.h"
#include "component_generic.h"
#include "API/Core/System/clanstring.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Label::CL_Label(
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(NULL)
{
	init("");
	find_preferred_size();
}

CL_Label::CL_Label(
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(NULL)
{
	init(text);
	find_preferred_size();
}

CL_Label::CL_Label(
	const CL_Point &pos,
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(CL_Rect(pos.x, pos.y, 0, 0), parent, style), impl(NULL)
{
	init(text);
	find_preferred_size();
}

CL_Label::CL_Label(
	const CL_Rect &pos,
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(pos, parent, style), impl(NULL)
{
	init(text);
}

void CL_Label::init(const std::string &text)
{
	impl = new CL_Label_Generic(this, text);
	get_style_manager()->connect_styles("label", this);
	set_focusable(false);
}

CL_Label::~CL_Label()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::string &CL_Label::get_text() const
{
	return impl->text;
}

CL_Origin CL_Label::get_alignment() const
{
	return impl->alignment;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_Label::set_text(const std::string &text)
{
	impl->text = text;
}

void CL_Label::set_text(int number)
{
	impl->text = CL_String::from_int(number);
}

void CL_Label::set_text(double number)
{
	impl->text = CL_String::from_double(number);
}

void CL_Label::clear()
{
	impl->text = "";
}

void CL_Label::set_alignment(CL_Origin alignment)
{
	impl->alignment = alignment;
}
