/*  $Id: menu_item.cpp,v 1.3 2003/12/15 19:44:15 harry Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "API/GUI/menu_item.h"
#include "API/GUI/stylemanager.h"
#include "API/Core/XML/dom_element.h"
#include "menu_item_generic.h"
#include "component_generic.h"
#include "API/Core/System/clanstring.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_MenuItem::CL_MenuItem(
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(NULL)
{
	impl = new CL_MenuItem_Generic(this, text);
	get_style_manager()->connect_styles("menu_item", this);

	set_focusable(false);

	find_preferred_size();
}

CL_MenuItem::~CL_MenuItem()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::string &CL_MenuItem::get_text() const
{
	return impl->text;
}

bool CL_MenuItem::is_toggling() const
{
	return impl->toggling;
}

bool CL_MenuItem::is_selected() const
{
	return impl->selected;
}

bool CL_MenuItem::use_icon() const
{
	return impl->use_icon;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_MenuItem::set_toggling(bool toggle)
{
	impl->toggling = toggle;
}

void CL_MenuItem::set_use_icon(bool use_icon)
{
	impl->use_icon = use_icon;
}

void CL_MenuItem::set_selected(bool selected)
{
	impl->selected = selected;
}

void CL_MenuItem::set_text(const std::string &text)
{
	impl->text = text;
}

void CL_MenuItem::set_text(int number)
{
	impl->text = CL_String::from_int(number);
}

void CL_MenuItem::set_text(double number)
{
	impl->text = CL_String::from_double(number);
}

void CL_MenuItem::clear()
{
	impl->text = "";
}
