/*  $Id: radiogroup.cpp,v 1.5 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "../API/GUI/radiogroup.h"
#include "radiogroup_generic.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_RadioGroup::CL_RadioGroup()
{
	impl = new CL_RadioGroup_Generic();
}

CL_RadioGroup::~CL_RadioGroup()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

int CL_RadioGroup::get_count() const
{
	return impl->buttons.size();
}
	
CL_RadioButton *CL_RadioGroup::get_toggled() const
{
	return impl->get_toggled();
}

const std::vector<CL_RadioButton *> &CL_RadioGroup::get_buttons() const
{
	return impl->buttons;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_RadioGroup::add(CL_RadioButton *button, bool delete_component)
{
	impl->add(button, delete_component);
}
	
void CL_RadioGroup::remove(CL_RadioButton *button)
{
	impl->remove(button);
}

void CL_RadioGroup::set_checked(CL_RadioButton *button)
{
	impl->set_checked(button);
}

/////////////////////////////////////////////////////////////////////////////
// Signals:

CL_Signal_v1<CL_RadioButton *> &CL_RadioGroup::sig_selection_changed()
{
	return impl->sig_selection_changed;
}
