/*  $Id: dcc_download.cpp,v 1.3 2003/09/24 13:00:34 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "API/Network/IRC/dcc_download.h"
#include "dcc_download_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_DCCDownload Construction:

CL_DCCDownload::CL_DCCDownload() : impl(0)
{
}

CL_DCCDownload::CL_DCCDownload(
	const std::string &server,
	const std::string &port,
	const std::string &filename,
	int total_size,
	CL_OutputSourceProvider *output,
	bool delete_provider) : impl(0)
{
	impl = new CL_DCCDownload_Generic(server, port, filename, total_size, output, delete_provider);
	impl->add_ref();
}

CL_DCCDownload::CL_DCCDownload(const CL_DCCDownload &copy) : impl(copy.impl)
{
	if (impl) impl->add_ref();
}
	
CL_DCCDownload::~CL_DCCDownload()
{
	if (impl) impl->release_ref();
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_DCCDownload Attributes:

int CL_DCCDownload::get_bytes_transfered()
{
	CL_MutexSection mutex_lock(&impl->mutex);
	return 0;
}
	
int CL_DCCDownload::get_total_size()
{
	CL_MutexSection mutex_lock(&impl->mutex);
	return impl->total_size;
}
	
CL_DCCDownload::Status CL_DCCDownload::get_status()
{
	CL_MutexSection mutex_lock(&impl->mutex);
	return impl->status;
}

CL_Signal_v1<const std::string &> &CL_DCCDownload::sig_connection_lost()
{
	return impl->sig_connection_lost;
}

CL_Signal_v0 &CL_DCCDownload::sig_download_complete()
{
	return impl->sig_download_complete;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DCCDownload Operations:

CL_DCCDownload &CL_DCCDownload::operator =(const CL_DCCDownload &copy)
{
	if (impl == copy.impl) return *this;
	if (impl) impl->release_ref();
	impl = copy.impl;
	if (impl) impl->add_ref();
	return *this;
}

void CL_DCCDownload::reconnect()
{
	impl->reconnect_trigger.set_flag();
}

/////////////////////////////////////////////////////////////////////////////
// CL_DCCDownload Implementation:
