/*  $Id: inputsource_socket.cpp,v 1.8 2004/04/24 20:50:50 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "socket_generic.h"
#include "API/Network/Socket/socket.h"
#include "API/Network/Socket/inputsource_socket.h"

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Socket construction:

CL_InputSource_Socket::CL_InputSource_Socket(const CL_Socket &sock)
: socket(sock.impl), ref_count_socket(true)
{
	socket->add_ref();
}

CL_InputSource_Socket::~CL_InputSource_Socket()
{
	if (ref_count_socket) socket->release_ref();
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Socket attributes:

int CL_InputSource_Socket::tell() const
{
	return 0;
}

int CL_InputSource_Socket::size() const
{
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Socket operations:

int CL_InputSource_Socket::read(void *data, int size)
{
	char *d = (char*) data;
	int gotten = 0;

	CL_Socket s(socket);
	while (size-gotten > 0)
	{
		int received = s.recv(d+gotten, size-gotten);
		if (received == 0)
			break;
		if (received < 0)
		{
			s.get_read_trigger()->wait();
			continue;
		}

		gotten += received;
	}
	return gotten;
}

void CL_InputSource_Socket::open()
{
}

void CL_InputSource_Socket::close()
{
}

CL_InputSource *CL_InputSource_Socket::clone() const
{
	return new CL_InputSource_Socket(CL_Socket(socket));
}

void CL_InputSource_Socket::seek(int pos, SeekEnum seek_type)
{
}

void CL_InputSource_Socket::push_position()
{
}

void CL_InputSource_Socket::pop_position()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputSource_Socket implementation:
