/*  $Id: ip_address_generic.h,v 1.8 2003/07/20 22:34:40 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef ip_address_generic_header
#define ip_address_generic_header

#include <string>
#include "API/Network/Socket/ip_address.h"

//: Socket name; container class for an IP address and port.
class CL_IPAddress_Generic
{
//! Construction:
public:
	//: Constructs an IP address using INADDR_ANY and port 0.
	CL_IPAddress_Generic() {;}

	//: Constructs an IP address using INADDR_ANY and the specified port.
	CL_IPAddress_Generic(const std::string &port) {;}

	//: Construct an IP address using the hostname and the port.
	CL_IPAddress_Generic(const std::string &hostname, const std::string &port) {;}

	//: Copy constructor.
	CL_IPAddress_Generic(const CL_IPAddress_Generic &copy) {;}

//! Attributes:
public:
	//: Returns the internal addrinfo struct
	virtual void get_addrinfo(int type, sockaddr &addr, int &len, int domain) const=0;

	//: Returns the IP address in network byte order.
	virtual std::string get_address() const=0;

	//: Returns the IP port in host byte order.
	virtual std::string get_port() const=0;

	//: Returns true if objects are the same.
	virtual bool operator == (const CL_IPAddress &other_instance) const=0;

	//: Returns true if the other address is less.
	//- <p>This is used for sorting
	//- purposes (eg. if you use a std::map<CL_IPAddress, CL_Socket>), and sorts
	//- the address based on lowest IP number address.</p>
	virtual bool operator < (const CL_IPAddress &other_instance) const=0;

	//: Returns true if the other address is greater.
	//- <p>This is used for sorting
	//- purposes (eg. if you use a std::map<CL_IPAddress, CL_Socket>), and sorts
	//- the address based on lowest IP number address.</p>
	virtual bool operator > (const CL_IPAddress &other_instance) const=0;

//! Operations:
public:
	//: Set the IP address using a hostname.
	//param hostname: Can be an IPv4 dotted-quad, hostname or a valid IPv6 address.
	virtual void set_address(const std::string &hostname, const std::string &port)=0;

	//: Set the IP port.
	virtual void set_port(const std::string &new_port)=0;

	//: Do a DNS lookup on the IP address.
	virtual std::string dns_lookup() const=0;
	
		//: Check to see if the address has an IPv4 mapping
	//: is_ipv4() and is_ipv6() are not mutally exclusive
	//: An address can have both IPv4 and IPv6 mappings
	virtual bool is_ipv4() const=0;
	
	//: Check to see if the address has an IPv6 mapping
	//: is_ipv4() and is_ipv6() are not mutally exclusive
	//: An address can have both IPv4 and IPv6 mappings
	virtual bool is_ipv6() const=0;
};

#endif
