/*  $Id: setupnetwork_unix.cpp,v 1.9 2004/03/13 23:04:13 mbn Exp $
	
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "API/Network/setupnetwork.h"
#include "Socket/event_trigger_socket.h"
#include "Socket/socket_select.h"
#include <signal.h>

static int ref_count = 0;
static sighandler_t old_handler = 0;

CL_SetupNetwork::CL_SetupNetwork(bool register_resources_only)
{
	CL_SetupNetwork::init(register_resources_only);
}

CL_SetupNetwork::~CL_SetupNetwork()
{
	CL_SetupNetwork::deinit();
}

/////////////////////////////////////////////////////////////////////////////
// Initialize network.

void CL_SetupNetwork::init(bool register_resources_only)
{
	ref_count++;
	if (ref_count > 1) return;

	// Dont make broken pipes terminate the program (doh):
	old_handler = signal(13, SIG_IGN); // 13 = SIGPIPE

	CL_EventTrigger_Socket::socket_select = new CL_SocketSelect();
}

/////////////////////////////////////////////////////////////////////////////
// Deinitialize network.

void CL_SetupNetwork::deinit()
{
	ref_count--;
	if (ref_count > 0) return;

	delete CL_EventTrigger_Socket::socket_select;
	CL_EventTrigger_Socket::socket_select = 0;
	
	signal(13, old_handler);
}
