/*  $Id: soundprovider_raw.cpp,v 1.3 2003/07/20 22:21:28 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Sound/precomp.h"
#include "API/Sound/SoundProviders/soundprovider_raw.h"
#include "soundprovider_raw_generic.h"
#include "soundprovider_raw_session.h"

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Raw construction:

CL_SoundProvider_Raw::CL_SoundProvider_Raw(
	void *sound_data,
	int num_samples,
	int bytes_per_sample,
	bool stereo,
	int frequency) : impl(new CL_SoundProvider_Raw_Generic)
{
	int data_size = num_samples * bytes_per_sample;
	if (stereo) data_size *= 2;

	impl->sound_data = new unsigned char[data_size];
	memcpy(impl->sound_data, sound_data, data_size);
	impl->num_samples = num_samples;
	impl->bytes_per_sample = bytes_per_sample;
	impl->stereo = stereo;
	impl->frequency = frequency;
}

CL_SoundProvider_Raw::~CL_SoundProvider_Raw()
{
	delete[] impl->sound_data;
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Raw operations:

CL_SoundProvider_Session *CL_SoundProvider_Raw::begin_session()
{
	return new CL_SoundProvider_Raw_Session(impl);
}

void CL_SoundProvider_Raw::end_session(CL_SoundProvider_Session *session)
{
	delete session;
}

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Raw implementation:
