/*  $Id: soundprovider_wave_session.cpp,v 1.3 2003/07/20 22:21:28 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Sound/precomp.h"
#include "soundprovider_wave_session.h"
#include "soundprovider_wave_generic.h"
#include "API/Sound/soundformat.h"

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Wave_Session construction:

CL_SoundProvider_Wave_Session::CL_SoundProvider_Wave_Session(CL_SoundProvider_Wave_Generic *data) :
	data(data), position(0)
{
}

CL_SoundProvider_Wave_Session::~CL_SoundProvider_Wave_Session()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Wave_Session attributes:

int CL_SoundProvider_Wave_Session::get_num_samples() const
{
	return data->num_samples;
}

int CL_SoundProvider_Wave_Session::get_frequency() const
{
	return data->frequency;
}

CL_SoundFormat CL_SoundProvider_Wave_Session::get_format() const
{
	return data->format;
}

int CL_SoundProvider_Wave_Session::get_num_channels() const
{
	return data->num_channels;
}

int CL_SoundProvider_Wave_Session::get_position() const
{
	return position;
}

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Wave_Session operations:

bool CL_SoundProvider_Wave_Session::eof() const
{
	return (position >= get_num_samples());
}

void CL_SoundProvider_Wave_Session::stop()
{
}

bool CL_SoundProvider_Wave_Session::play()
{
	return true;
}
	
bool CL_SoundProvider_Wave_Session::set_position(int pos)
{
	position = pos;
	return true;
}

int CL_SoundProvider_Wave_Session::get_data(void **data_ptr, int data_requested)
{
	int num_samples = get_num_samples();
	if (position + data_requested > num_samples)
	{
		data_requested = num_samples - position;
		if (data_requested < 0) return 0;
	}

	if (data->format == sf_16bit_signed)
	{
		short **channels = (short **) data_ptr;

		if (data->num_channels == 2)
		{
			short *src = (short *) data->data + position * 2;
			for (int i=0; i<data_requested; i++)
			{
				channels[0][i] = *(src++);
				channels[1][i] = *(src++);
			}
		}
		else
		{
			memcpy(channels[0], data->data+position*2, data_requested*2);
		}
	}
	else if (data->format == sf_8bit_signed)
	{
		char **channels = (char **) data_ptr;

		if (data->num_channels == 2)
		{
			char *src = (char *) data->data + position * 2;
			for (int i=0; i<data_requested; i++)
			{
				channels[0][i] = *(src++);
				channels[1][i] = *(src++);
			}
		}
		else
		{
			memcpy(channels[0], data->data+position, data_requested);
		}
	}

	position += data_requested;
	return data_requested;
}

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProvider_Wave_Session implementation:
