/*  $Id: setupsound.cpp,v 1.4 2004/03/13 23:04:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Sound/precomp.h"
#include "API/Sound/setupsound.h"
#include "API/Sound/SoundProviders/soundprovider_wave.h"
#include "API/Sound/SoundProviders/soundprovider_type_register.h"
#include "API/Core/Resources/resource_manager.h"
#include "resourcetype_sample.h"

/////////////////////////////////////////////////////////////////////////////
// CL_SetupSound operations:

static int ref_count = 0;
static CL_Slot slot_resource_added;
static CL_SoundProviderType *providertype_wave = 0;

static void clansound_resource_added(CL_Resource &resource)
{
	std::string type = resource.get_type();

	if (type == "sample") new CL_ResourceData_Sample(resource);
}

CL_SetupSound::CL_SetupSound(bool register_resources_only)
{
	CL_SetupSound::init(register_resources_only);
}

CL_SetupSound::~CL_SetupSound()
{
	CL_SetupSound::deinit();
}

void CL_SetupSound::init(bool register_resources_only)
{
	ref_count++;
	if (ref_count > 1) return;

	slot_resource_added = CL_ResourceManager::sig_resource_added().connect(&clansound_resource_added);
	providertype_wave = new CL_SoundProviderType_Register<CL_SoundProvider_Wave>("wav");

	if (register_resources_only) return;
}

void CL_SetupSound::deinit()
{
	ref_count--;
	if (ref_count > 0) return;	// Wait until final call to deinit

	delete providertype_wave;
	providertype_wave = 0;
	slot_resource_added = CL_Slot();
}
