/*  $Id: soundbuffer_session_generic.h,v 1.9 2004/02/01 20:28:56 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_soundbuffer_session_generic
#define header_soundbuffer_session_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include <vector>
#include "API/Core/System/mutexsharedptr.h"
#include "API/Core/System/mutex.h"
#include "API/Sound/soundformat.h"

class CL_SoundFilter;
class CL_SoundBuffer_Generic;
class CL_SoundProvider_Session;
class CL_SoundOutput_Generic;

class CL_SoundBuffer_Session_Generic
{
//! Construction:
public:
	CL_SoundBuffer_Session_Generic(
		CL_MutexSharedPtr<CL_SoundBuffer_Generic> soundbuffer,
		bool looping,
		CL_SoundOutput_Generic *output);
	
	virtual ~CL_SoundBuffer_Session_Generic();

//! Attributes:
public:
	CL_MutexSharedPtr<CL_SoundBuffer_Generic> soundbuffer;

	CL_SoundProvider_Session *provider_session;

	CL_SoundOutput_Generic *output;

	float volume;
	
	float pan;
	
	bool looping;
	
	bool playing;
	
	std::vector<CL_SoundFilter *> filters;
	
	std::vector<bool> delete_filters;
	
	mutable CL_Mutex mutex;

//! Operations:
public:
	bool mix_to(int **sample_data, int **temp_data, int num_samples, int channels);

//! Implementation:
private:
	//: Fills temporary buffers with data from provider.
	void get_data();

	//: Temporary channel buffers containing sound data in provider frequency.
	int **buffer_data;

	//: Size of temporary channel buffers.
	int num_buffer_samples;

	//: Number of temporary channel buffers;
	int num_buffer_channels;

	//: Current playback position in temporary buffers.
	double buffer_position;

	//: Number of samples currently written to buffer_data.
	int buffer_samples_written;
};

#endif
