/*  $Id: soundoutput_generic.h,v 1.7 2004/02/01 20:28:56 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_soundoutput_generic
#define header_soundoutput_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include <vector>
#include <string>
#include <list>
#include "API/Core/System/thread.h"
#include "API/Core/System/mutex.h"
#include "API/Core/System/event_trigger.h"
#include "API/Core/System/mutexsharedptr.h"

class CL_SoundFilter;
class CL_SoundBuffer_Session_Generic;

class CL_SoundOutput_Generic
{
//! Construction:
public:
	CL_SoundOutput_Generic(int mixing_frequency);
	
	virtual ~CL_SoundOutput_Generic();

//! Attributes:
public:
	std::string name;

	int mixing_frequency;

	float volume;

	float pan;

	std::vector<CL_SoundFilter *> filters;

	std::vector<bool> delete_filters;

	CL_Thread thread;

	CL_EventTrigger stop_mixer;

	std::list< CL_MutexSharedPtr<CL_SoundBuffer_Session_Generic> > sessions;

	CL_Mutex mutex;

//! Operations:
public:
	void add_ref();
	
	void release_ref();

	void play_session(CL_MutexSharedPtr<CL_SoundBuffer_Session_Generic> session);

	void stop_session(CL_MutexSharedPtr<CL_SoundBuffer_Session_Generic> session);

protected:
	//: Worker thread for output device. Mixes the audio and sends it to write_fragment.
	void mixer_thread();
	
	//: Called when we have no samples to play - and wants to tell the soundcard
	//: about this possible event.
	virtual void silence() = 0;

	//: Returns the buffer size used by device (returned as num [stereo] samples).
	virtual int get_fragment_size() = 0;

	//: Writes a fragment to the soundcard.
	virtual void write_fragment(short *data) = 0;

	//: Waits until output source isn't full anymore.
	virtual void wait() = 0;

//! Implementation:
private:

	int ref_count;
};

#endif
