/*  $Id: soundprovider_vorbis_session.h,v 1.3 2003/10/14 15:52:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_sound_provider_vorbis_session
#define header_sound_provider_vorbis_session

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Sound/soundprovider_session.h"
#include "vorbis/codec.h"

class CL_SoundProvider_Vorbis_Generic;
class CL_InputSource;

class CL_SoundProvider_Vorbis_Session : public CL_SoundProvider_Session
{
//! Construction:
public:
	CL_SoundProvider_Vorbis_Session(CL_SoundProvider_Vorbis_Generic *data);

	~CL_SoundProvider_Vorbis_Session();

//! Attributes:
public:
	virtual int get_num_samples() const;

	virtual int get_frequency() const;

	virtual CL_SoundFormat get_format() const;

	virtual int get_num_channels() const;

	virtual int get_position() const;

//! Operations:
public:
	virtual bool eof() const;
	
	virtual void stop();

	virtual bool play();
		
	virtual bool set_position(int pos);
	
	virtual int get_data(void **data_ptr, int data_requested);

//! Implementation:
private:
	void stream_data();

	CL_SoundProvider_Vorbis_Generic *data;

	int num_samples;

	int position;

	CL_InputSource *input;

	bool stream_eof;

	//: Sync and verify incoming physical bitstream.
	ogg_sync_state oy;

	//: Take physical pages, weld into a logical stream of packets.
	ogg_stream_state os;

	//: One Ogg bitstream page.  Vorbis packets are inside.
	ogg_page og;

	//: One raw packet of data for decode.
	ogg_packet op;

	//: Struct that stores all the static vorbis bitstream settings.
	vorbis_info vi;

	//: Struct that stores all the bitstream user comments.
	vorbis_comment vc;

	//: Central working state for the packet->PCM decoder.
	vorbis_dsp_state vd;

	//: Local working space for packet->PCM decode.
	vorbis_block vb;
};

#endif
