/*
	$Id: resourceoptions.h,v 1.1 2003/06/04 12:27:24 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

#ifndef RESOURCEOPTIONS_H_
#define RESOURCEOPTIONS_H_

#include <map>
#include <string>
#include "resourceoption.h"

//: Resource options of a resource.
class ResourceOptions
{
//! Construction:
public:
	//: Resource Options constructor.
	ResourceOptions();

	//: Resource Options destructor.
	~ResourceOptions();

//! Attributes:
public:
	//: Returns a list of all the resource options.
	std::map<std::string, ResourceOption> &get_options();

	//: Returns a resource option.
	ResourceOption &get_option(const std::string &name);

	const ResourceOption &get_option(const std::string &name) const;

	//: Returns true if an option exists.
	bool exists(const std::string &name);

//! Operations:
public:
	//: Add a resource option.
	void add(const ResourceOption &option);
	void remove(const std::string&);

//! Implementation:
private:
	std::map<std::string, ResourceOption> options;
};

#endif
